/////////////////////////////////////////////////////////////////////////
///@author 中泰证券股份有限公司
///@file xtp_api_struct.h
///@brief 定义业务数据结构
/////////////////////////////////////////////////////////////////////////
#ifndef _XTP_API_STRUCT_H_
#define _XTP_API_STRUCT_H_

#include "xtp_api_struct_common.h"
#include "xquote_api_struct.h"
#include "xoms_api_struct.h"
#include "xoms_api_fund_struct.h"
#include "xquote_api_rebuild_tbt_struct.h"

#endif // !_XTP_API_STRUCT_H_

